import threading
import time
from ExtraDecorators import validatetyping

class RateLimiter:
    @validatetyping
    def __init__(self, threshold:int,upperCap:int, decay_rate:int, decay_time:int, extreme_case:bool=False)->None:
        '''
        threshold: int -> The amount of triggers before the rate limiter will set hit to True
        upperCap: int -> The maximum amount of triggers to count (if reached any aditional triggers will not surpass the upperCap)
        decay_rate: int -> The amount of triggers that will be removed from the counter every decay cycle
        decay_time: int -> The time between each decay cycle
        extreme_case: bool -> if True, the RateLimited exception will be raised when the threshold is reached if False, only rate_limiter.hit will be set to True but no exception will be raised
        '''        
        self.threshold = threshold
        self.uppercap = upperCap
        self.decay_rate = decay_rate
        self.decay_time = decay_time
        self.extreme_case = extreme_case
        self.triggered = 0
        self.stoped = False
        self.lock = threading.Lock()
        self.decaying = True
        self.hit = False
        
        # Start decay thread
        decay_thread = threading.Thread(target=self.__decay)
        decay_thread.daemon = True
        decay_thread.start()

    def increment(self):
        with self.lock:
            if self.triggered < self.uppercap:
                self.triggered += 1
            if self.triggered > self.threshold:
                self.hit = True
                if self.extreme_case:
                    raise RateLimited()
            
    def __decay(self):
        while not self.stoped:
            time.sleep(self.decay_time)
            if not self.decaying:
                continue
            with self.lock:
                self.triggered -= self.decay_rate
                if self.triggered < self.threshold:
                    self.hit = False
                if self.triggered < 0:
                    self.triggered = 0
        
    def pause_decay(self):
        if self.stoped:
            raise ValueError("Decay is already stoped!")
        self.decaying = False
    
    def resume_decay(self):
        if self.stoped:
            raise ValueError("A stopped decay can't be resumed! Use ")
        self.decaying = True


class RateLimited(Exception):
    def __init__(self, message="Rate limit reached"):
        self.message = message
        super().__init__(self.message)


from setuptools import setup, find_packages

setup(
    name='extrautilities',
    version='0.6',
    packages=find_packages(),
    description='This package provides a few extra utilities for Python, like a "RateLimiter" class.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='RandomTimeTV',
    author_email='dergepanzerte1@gmail.com',
    license='Unlicensed',
    url='https://github.com/RandomTimeLP/ExtraUtils',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
    keywords='RateLimit, asyncTokens',
    install_requires=["extradecorators", "cryptography"],
)


from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.serialization import load_pem_public_key


def gen_keypair():
    priv_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048,
        backend=default_backend()
    )
    pub_key = priv_key.public_key()
    return (priv_key, pub_key)

def encrypt(nachricht, pub_key):
    message = pub_key.encrypt(
        nachricht.encode(),
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None
        )
    )
    return message

def serial(key):
    return key.public_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PublicFormat.SubjectPublicKeyInfo
    ).decode('utf-8')

def load(key):
    pub_key = load_pem_public_key(
        key.encode('utf-8'),
        backend=default_backend()
    )
    return pub_key

def decrypt(message, priv_key):
    decrypted_message = priv_key.decrypt(
        message,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None
        )
    )
    return decrypted_message.decode()

# Example:
# priv_key, pub_key = generiere_rsa_schluesselpaar()
# message = encrypt("Geheime Nachricht", pub_key)
# print(f"Verschlüsselt: {message}")
# 
# decrypted_message = decrypt(message, priv_key)
# print(f"Entschlüsselt: {decrypted_message}")